﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Activities;
using Microsoft.TeamFoundation.Build.Client;

namespace Benday.TeamBuild2012.Activities
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public sealed class IisCreateWebApplication : BuildActivityBase
    {
        public override string ActivityName
        {
            get { return "IisCreateWebApplication"; }
        }

        private IisUtility m_IisUtilityInstance;
        public IisUtility IisUtilityInstance
        {
            get
            {
                if (m_IisUtilityInstance == null)
                {
                    m_IisUtilityInstance = new IisUtility();
                }

                return m_IisUtilityInstance;
            }
        }

        [RequiredArgument]
        public InArgument<string> SiteName { get; set; }

        [RequiredArgument]
        public InArgument<string> AppPoolName { get; set; }

        [RequiredArgument]
        public InArgument<string> ApplicationName { get; set; }

        [RequiredArgument]
        public InArgument<string> ApplicationPath { get; set; }

        [RequiredArgument]
        public InArgument<bool> SetAuthenticationSettings { get; set; }

        [RequiredArgument]
        public InArgument<bool> AuthenticationAnonymous { get; set; }

        [RequiredArgument]
        public InArgument<bool> AuthenticationWindows { get; set; }

        [RequiredArgument]
        public InArgument<bool> AuthenticationBasic { get; set; }

        private string VerifySiteName(string siteName)
        {
            if (String.IsNullOrEmpty(siteName) == true)
            {
                var sites = IisUtilityInstance.GetSites();

                if (sites == null || sites.Count < 1)
                {
                    throw new InvalidOperationException("Could not determine default site name because IIS does not contain any sites.");
                }
                else
                {
                    return sites[0];
                }
            }
            else
            {
                return siteName;
            }
        }

        private void UpdateSecuritySettings(CodeActivityContext context, string siteName, string applicationName)
        {
            bool doSetAuthenticationSettings = SetAuthenticationSettings.Get(context);

            if (doSetAuthenticationSettings == false)
            {
                LogDebugMessage(context, "Skipped update to security settings.");                    
            }
            else
            {
                LogDebugMessage(context, "Setting security settings for web application.");

                var allowAnonymous = AuthenticationAnonymous.Get(context);
                var allowWindows = AuthenticationWindows.Get(context);
                var allowBasic = AuthenticationBasic.Get(context);

                LogDebugVariableValue(context, "AllowAnonymous", allowAnonymous);
                LogDebugVariableValue(context, "AllowBasic", allowBasic);
                LogDebugVariableValue(context, "AllowWindows", allowWindows);

                SecuritySettings settings = new SecuritySettings() 
                    { Anonymous = allowAnonymous, Basic = allowBasic, Windows = allowWindows };

                IisUtilityInstance.SetSecurityForApplication(siteName, applicationName, settings);

                LogDebugMessage(context, "Security settings for web application set.");
            }
        }

        private void UpdateAppPoolSettings(CodeActivityContext context, string tempAppPoolName, string tempSiteName, string tempApplicationName)
        {
            if (String.IsNullOrEmpty(tempAppPoolName) == false)
            {
                LogDebugMessage(context, "Setting app pool for web application.");

                IisUtilityInstance.SetAppPoolForApplication(tempSiteName,
                    tempApplicationName,
                    tempAppPoolName);

                LogDebugMessage(context, "App pool for web application set.");
            }
            else
            {
                LogDebugMessage(context, "Skipped setting app pool.");
            }
        }

        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                LogDebugMessage(context, "Initializing...");

                string tempAppPoolName = AppPoolName.Get(context);
                string tempSiteName = SiteName.Get(context);
                string tempApplicationName = ApplicationName.Get(context);
                string tempApplicationPath = ApplicationPath.Get(context);

                LogDebugVariableValue(context, "AppPoolName", tempAppPoolName);
                LogDebugVariableValue(context, "SiteName", tempSiteName);
                LogDebugVariableValue(context, "ApplicationName", tempApplicationName);
                LogDebugVariableValue(context, "ApplicationPath", tempApplicationPath);

                tempSiteName = VerifySiteName(tempSiteName);

                IisUtilityInstance.CreateApplication(tempSiteName, tempApplicationName, tempApplicationPath);

                UpdateAppPoolSettings(context, tempAppPoolName, tempSiteName, tempApplicationName);
                UpdateSecuritySettings(context, tempSiteName, tempApplicationName);

                LogDebugMessage(context, String.Format("Created web application for app pool '{0}', site name: '{1}', application name: '{2}' at application path: '{3}'",
                        tempAppPoolName, tempSiteName, tempApplicationName, tempApplicationPath));

                LogUsefulMessage(context, "Created web application."); 
            }
            catch (Exception ex)
            {
                LogException(context, ex);

                throw;
            }
        }
    }
}
