
using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.TeamBuild2012;
using System.IO;
using Benday.TeamBuild2012.Activities;

namespace Benday.TeamBuild2012.UnitTests
{
    [TestClass]
    public class IisUtilityFixture : UnitTestBase
    {
        public IisUtilityFixture()
        {
        }

        private string GetPathToTestWebApp()
        {
            string path = TestContext.TestDir;

            Assert.IsTrue(Directory.Exists(path), "Path does not exist.  '{0}'", path);

            return path;
        }

        [TestMethod]
        public void IisUtility_GetSites()
        {
            IList<string> siteNames = UtilityInstance.GetSites();

            Assert.IsNotNull(siteNames, "Site names collection was null.");
            Assert.AreNotEqual<int>(0, siteNames.Count, "Site count.");

            foreach (var item in siteNames)
            {
                Console.WriteLine(item);
            }
        }

        [TestMethod]
        public void IisUtility_GetApplicationsForDefaultSite()
        {
            IList<string> applicationNames = UtilityInstance.GetApplications(STR_DefaultWebSite);

            Assert.IsNotNull(applicationNames, "Application names collection was null.");
            Assert.AreNotEqual<int>(0, applicationNames.Count, "Application count.");

            foreach (var item in applicationNames)
            {
                Console.WriteLine(item);
            }
        }

        private void VerifyApplicationDoesNotExist(string siteName, string applicationName)
        {
            Assert.IsFalse(UtilityInstance.ApplicationExists(siteName, applicationName), "Application should not exist.");
        }

        [TestMethod]
        public void IisUtility_CreateApplicationAndVerifyAndDelete()
        {
            string applicationName = "AppName";

            DeleteApplicationIfItExists(STR_DefaultWebSite, applicationName);
            VerifyApplicationDoesNotExist(STR_DefaultWebSite, applicationName);

            UtilityInstance.CreateApplication(STR_DefaultWebSite, applicationName, GetPathToTestWebApp());

            Assert.IsTrue(UtilityInstance.ApplicationExists(STR_DefaultWebSite, applicationName), 
                "Application should exist after create.");

            DeleteApplicationIfItExists(STR_DefaultWebSite, applicationName);
        }

        [TestMethod]
        public void IisUtility_ApplicationExists_False()
        {
            Assert.IsFalse(UtilityInstance.ApplicationExists(
                STR_DefaultWebSite, Guid.NewGuid().ToString()), "Application should not exist.");
        }

        private void CreateApplicationIfItDoesNotExist(string website, string webapplication)
        {
            if (UtilityInstance.ApplicationExists(website, webapplication) == false)
            {
                UtilityInstance.CreateApplication(website, webapplication, GetPathToTestWebApp());
            }
        }
        [TestMethod]
        public void IisUtility_ApplicationExists_True_WithoutLeadingSlash()
        {
            CreateApplicationIfItDoesNotExist(STR_DefaultWebSite, STR_TestWebApplication);
            
            Assert.IsTrue(UtilityInstance.ApplicationExists(STR_DefaultWebSite, STR_TestWebApplication), "Application should exist.");
        }

        [TestMethod]
        public void IisUtility_ApplicationExists_True_WithLeadingSlash()
        {
            CreateApplicationIfItDoesNotExist(STR_DefaultWebSite, STR_TestWebApplication);

            Assert.IsTrue(UtilityInstance.ApplicationExists(STR_DefaultWebSite, STR_TestWebApplication), "Application should exist.");
        }

        [TestMethod]
        public void IisUtility_SiteExists_True()
        {
            Assert.IsTrue(UtilityInstance.SiteExists(STR_DefaultWebSite), "Site should exist.");
        }

        [TestMethod]
        public void IisUtility_SiteExists_False()
        {
            Assert.IsFalse(UtilityInstance.SiteExists(Guid.NewGuid().ToString()), "Site should not exist.");
        }

        [TestMethod]
        public void IisUtility_GetAppPools()
        {
            IList<string> appPoolNames = UtilityInstance.GetAppPools();

            Assert.IsNotNull(appPoolNames, "AppPool names collection was null.");
            Assert.AreNotEqual<int>(0, appPoolNames.Count, "AppPool count.");

            foreach (var item in appPoolNames)
            {
                Console.WriteLine(item);
            }
        }

        [TestMethod]
        public void IisUtility_AppPoolExists()
        {
            Assert.IsTrue(UtilityInstance.AppPoolExists("ASP.NET v4.0"), "AppPool does not exist.");
            Assert.IsFalse(UtilityInstance.AppPoolExists(Guid.NewGuid().ToString()), "AppPool should not exist.");
        }

        [TestMethod]
        public void IisUtility_SetApplicationAppPool()
        {
            string applicationName = "AppName";

            DeleteApplicationIfItExists(STR_DefaultWebSite, applicationName);
            VerifyApplicationDoesNotExist(STR_DefaultWebSite, applicationName);

            UtilityInstance.CreateApplication(STR_DefaultWebSite, applicationName, GetPathToTestWebApp());

            Assert.IsTrue(UtilityInstance.ApplicationExists(STR_DefaultWebSite, applicationName),
                "Application should exist after create.");

            string originalAppPool = UtilityInstance.GetAppPoolForApplication(STR_DefaultWebSite, applicationName);
            string expectedAppPool = "TestAppPool";

            UtilityInstance.SetAppPoolForApplication(STR_DefaultWebSite, applicationName, expectedAppPool);

            string actualAppPool = UtilityInstance.GetAppPoolForApplication(STR_DefaultWebSite, applicationName);

            Assert.AreEqual<string>(expectedAppPool, actualAppPool, "App pool didn't update.");

            DeleteApplicationIfItExists(STR_DefaultWebSite, applicationName);
        }

        private void SetSecuritySettingsAndAssert(string siteName, string applicationName, bool expectedAnonymous, bool expectedBasic, bool expectedWindows)
        {
            SecuritySettings updated = new SecuritySettings();

            updated.Anonymous = expectedAnonymous;
            updated.Windows = expectedWindows;
            updated.Basic = expectedBasic;

            UtilityInstance.SetSecurityForApplication(siteName, applicationName, updated);

            AssertSecuritySettings(siteName, applicationName, expectedAnonymous, expectedBasic, expectedWindows);
        }

        private void AssertSecuritySettings(string siteName, string applicationName, bool expectedAnonymous, bool expectedBasic, bool expectedWindows)
        {
            Assert.IsTrue(UtilityInstance.ApplicationExists(STR_DefaultWebSite, applicationName),
                "Application should exist.");

            SecuritySettings actual = UtilityInstance.GetSecurityForApplication(siteName, applicationName);

            Assert.AreEqual<bool>(expectedAnonymous, actual.Anonymous, "Anonymous");
            Assert.AreEqual<bool>(expectedWindows, actual.Windows, "Windows");
            Assert.AreEqual<bool>(expectedBasic, actual.Basic, "Basic");
        }

        [TestMethod]
        public void IisUtility_SetSecurity()
        {
            string applicationName = "AppName";

            DeleteApplicationIfItExists(STR_DefaultWebSite, applicationName);
            VerifyApplicationDoesNotExist(STR_DefaultWebSite, applicationName);

            UtilityInstance.CreateApplication(STR_DefaultWebSite, applicationName, GetPathToTestWebApp());

            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, true, false, false);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, false, true, false);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, false, false, true);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, true, true, false);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, true, false, true);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, false, false, false);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, true, true, true);
            SetSecuritySettingsAndAssert(STR_DefaultWebSite, applicationName, false, true, true);

            DeleteApplicationIfItExists(STR_DefaultWebSite, applicationName);
        }
    }
}
